@@ -34,7 +34,7 @@ class Sampler():
         use_global (bool, optional): Whether to use global sampler. Defaults to True.
         logging (bool, optional): Whether to log the training process. Defaults to True.
         nf_variable (None, optional): Mean and variance variables for the NF model. Defaults to None.
-        keep_quantile (float, optional): Quantile of chains to keep when training the normalizing flow model. Defaults to 0.5.
+        keep_quantile (float, optional): Quantile of chains to keep when training the normalizing flow model. Defaults to 0..
         local_autotune (None, optional): Auto-tune function for the local sampler. Defaults to None.
 
     """
@@ -62,6 +62,7 @@ class Sampler():
         nf_variable=None,
         keep_quantile=0,
         local_autotune=None,
+        train_thinning = 1,
     ):
         rng_key_init, rng_keys_mcmc, rng_keys_nf, init_rng_keys_nf = rng_key_set
 
@@ -87,6 +88,7 @@ class Sampler():
         self.use_global = use_global
         self.logging = logging
 
+
         self.nf_model = nf_model
         model_init = nf_model.init(init_rng_keys_nf, jnp.ones((1, self.n_dim)))
         params = model_init["params"]
@@ -95,12 +97,17 @@ class Sampler():
             self.variables = self.variables
 
         self.keep_quantile = keep_quantile
+        self.train_thinning = train_thinning
         self.nf_training_loop, train_epoch, train_step = make_training_loop(
             self.nf_model
         )
         self.global_sampler = make_nf_metropolis_sampler(self.nf_model)
 
         tx = optax.adam(self.learning_rate, self.momentum)
+        # tx = optax.chain(
+        #     optax.adaptive_grad_clip(1, eps=0.001),
+        #     optax.adam(self.learning_rate, self.momentum)
+        # )
         self.state = train_state.TrainState.create(
             apply_fn=nf_model.apply, params=params, tx=tx
         )
@@ -182,8 +189,8 @@ class Sampler():
 
         if self.use_global == True:
             if training == True:
-                positions = self.summary['training']['chains']
-                log_prob_output = self.summary['training']['log_prob']
+                positions = self.summary['training']['chains'][:,::self.train_thinning]
+                log_prob_output = self.summary['training']['log_prob'][:,::self.train_thinning]
 
                 if self.keep_quantile > 0:
                     max_log_prob = jnp.max(log_prob_output, axis=1)
@@ -246,7 +253,7 @@ class Sampler():
                 self.summary[summary_mode]['global_accs'], global_acceptance, axis=1
             )
 
-        last_step = positions[:, -1]
+        last_step = self.summary[summary_mode]['chains'][:, -1]
 
         return last_step
 
@@ -286,7 +293,7 @@ class Sampler():
             last_step = self.sampling_loop(last_step, training=True)
         return last_step
 
-    def production_run(self, initial_position: jnp.ndarray):
+    def production_run(self, initial_position: jnp.ndarray) -> jnp.array:
         """
         Sampling procedure that produce the final set of samples.
         The main difference between this and the global tuning step is
@@ -294,9 +301,11 @@ class Sampler():
         The data is stored in the summary dictionary.
         
         """
+        print("Starting Production run")
         last_step = initial_position
         for _ in range(self.n_loop_production):
-            self.sampling_loop(last_step)
+            last_step = self.sampling_loop(last_step)
+        return last_step
 
     def get_sampler_state(self, training: bool=False) -> dict:
         """
